@@ -16,22 +16,16 @@
 import argparse
 import logging
 import os
+import pkg_resources
 import subprocess
 import sys
 
 log = logging.getLogger("glean-install")
 
 
-def _find_gleansh_path():
-    # the "glean.sh" file is installed in /usr/bin/ on Fedora, and
-    # /usr/local/bin on Ubuntu/Debian thanks to differences in pip and
-    # where it likes to put scripts.
-    if os.path.exists("/usr/local/bin/glean.sh"):
-        return "/usr/local/bin"
-    if os.path.exists("/usr/bin/glean.sh"):
-        return "/usr/bin"
-    log.error("Unable to find glean.sh!")
-    sys.exit(1)
+def _find_scripts_dir():
+    p = pkg_resources.resource_filename(__name__, "init")
+    return p
 
 
 def install(source_file, target_file, mode='0755', replacements=dict()):
@@ -49,8 +43,7 @@ def install(source_file, target_file, mode='0755', replacements=dict()):
 
     log.info("Installing %s -> %s" % (source_file, target_file))
 
-    script_dir = os.path.join(
-        os.path.dirname(os.path.realpath(__file__)), 'init')
+    script_dir = _find_scripts_dir()
 
     cmd = ('install -D -g root -o root'
            ' -m {mode} {source_file} {target_file}').format(
@@ -85,20 +78,44 @@ def main():
                         action="store_true")
     parser.add_argument("-q", "--quiet", help="Be very quiet",
                         action="store_true")
+    # NOTE(dtantsur): there may be two reasons to disable the fallback:
+    # 1) Remote edge deployments where DHCP (if present at all) is likely
+    # incorrect and should not be used.
+    # 2) Co-existing with another tool that handles DHCP differently (IPv6,
+    # SLAAC) or does a more sophisticated configuration (like NetworkManager
+    # or, in case of ironic-python-agent, dhcp-all-interfaces, which is **not**
+    # recommended, but is done nonetheless, mostly for legacy reasons).
+    parser.add_argument("--no-dhcp-fallback", action="store_true",
+                        help="Do not fall back to DHCP. If this is on, "
+                             "something else must configure networking or "
+                             "it will be left unconfigured.")
 
     args = parser.parse_args()
-    p = _find_gleansh_path()
+    p = _find_scripts_dir()
+    extra_args = '--no-dhcp-fallback' if args.no_dhcp_fallback else ''
 
     if args.quiet:
         logging.basicConfig(level=logging.ERROR)
     else:
         logging.basicConfig(level=logging.INFO)
 
+    replacements = {
+        'INTERP': sys.executable,
+        'GLEAN_SCRIPTS_DIR': p,
+        'EXTRA_ARGS': extra_args,
+    }
+
+    # Write the path of the currently executing interpreter into the
+    # scripts dir.  This means glean shell scripts can call the
+    # sibling python-glean and know that it's using the glean we
+    # installed, even in a virtualenv etc.
+    install('python-glean.template', os.path.join(p, 'python-glean'),
+            mode='0755', replacements=replacements)
+
     # needs to go first because gentoo can have systemd along side openrc
     if os.path.exists('/etc/gentoo-release'):
         log.info('installing openrc services')
-        install('glean.openrc', '/etc/init.d/glean',
-                replacements={'GLEANSH_PATH': p})
+        install('glean.openrc', '/etc/init.d/glean', replacements=replacements)
     # Needs to check for the presence of systemd and systemctl
     # as apparently some packages may stage systemd init files
     # when systemd is not present.
@@ -106,19 +123,23 @@ def main():
     # may be wrong as install is generally executed in a chroot
     # with diskimage-builder.
 
-    if (os.path.exists('/usr/lib/systemd/system') and
-            (os.path.exists('/usr/bin/systemctl') or
-             os.path.exists('/bin/systemctl'))):
+    if (os.path.exists('/usr/lib/systemd/system')
+            and (os.path.exists('/usr/bin/systemctl')
+                 or os.path.exists('/bin/systemctl'))):
 
         log.info("Installing systemd services")
-        log.info("glean.sh in %s" % p)
 
+        log.info("Install early service")
+        install(
+            'glean-early.service',
+            '/usr/lib/systemd/system/glean-early.service',
+            mode='0644', replacements=replacements)
+        subprocess.call(['systemctl', 'enable', 'glean-early.service'])
         if os.path.exists('/etc/gentoo-release'):
             install(
                 'glean-networkd.service',
                 '/lib/systemd/system/glean.service',
-                mode='0644',
-                replacements={'GLEANSH_PATH': p})
+                mode='0644', replacements=replacements)
             subprocess.call(['systemctl', 'enable', 'glean.service'])
         else:
             log.info("Installing %s NetworkManager support" %
@@ -130,27 +151,40 @@ def main():
             install(
                 service_file,
                 '/usr/lib/systemd/system/glean@.service',
-                mode='0644',
-                replacements={'GLEANSH_PATH': p})
+                mode='0644', replacements=replacements)
         install(
             'glean-udev.rules',
             '/etc/udev/rules.d/99-glean.rules',
             mode='0644')
         if args.use_nm:
-            # glean handles resolv.conf, so this turns off nm touching
-            # it
-            install('nm-no-resolv-handling.conf',
-                    '/etc/NetworkManager/conf.d/nm-no-resolv-handling.conf',
-                    mode='0644')
+            # NetworkManager has a "after" network-pre, and
+            # glean@<interface> services have a "before".  However, if
+            # udev has not yet triggered and started the glean
+            # service, which it seems can be quite common in a slow
+            # environment like a binary-translated nested-vm, systemd
+            # may think it is fine to start NetworkManager because
+            # network-pre has been reached with no blockers.  Thus we
+            # override NetworkManager to wait for udev-settle, which
+            # should ensure the glean service has started; which will
+            # block network-pre until it finishes writing out the
+            # configs.
+            install(
+                'nm-udev-settle.override',
+                '/etc/systemd/system/NetworkManager.service.d/override.conf',
+                mode='0644')
+
     elif os.path.exists('/etc/init'):
         log.info("Installing upstart services")
-        install('glean.conf', '/etc/init/glean.conf')
+        install('glean.conf', '/etc/init/glean.conf',
+                replacements=replacements)
     elif os.path.exists('/sbin/rc-update'):
         subprocess.call(['rc-update', 'add', 'glean', 'boot'])
     else:
         log.info("Installing sysv services")
-        install('glean.init', '/etc/init.d/glean')
+        install('glean.init', '/etc/init.d/glean',
+                replacements=replacements)
         os.system('update-rc.d glean defaults')
 
+
 if __name__ == '__main__':
     main()
